#include <string>
#include <iostream>
#include <vector>
#include <algorithm>

#include "mex.h"
#include "Node.h"
#include "buildTree.h"
#include "ModelChecker.h"
#include "matrix.h"

#ifdef MODEL_THROMBIN
	#include "../../models/thrombin/thrombin_ode.h"
#elif MODEL_SEGCLOCK
	#include "../../models/segmentationclock/seg_clock_ode.h"
#elif MODEL_EGFNGF
	#include "../../models/egf-ngf/egf_ngf_ode.h"
#elif MODEL_REPRESS
	#include "../../models/repressilator/repressilator_ode.h"
#elif MODEL_TLR
	#include "../../models/TLRpathway/TLR_ode.h"
#endif

#include "cvode_sim.c"

using namespace std;

mxArray * getMexArray(const std::vector<int>& v);


extern void _main();

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
	double *p_in;
	double *x_in;
	double *x_pre,*x_post;
	//double *t_post;
	double dt;
    double pathway_identifier;
	int n_x_in, n_formulas_in, i, n, n_p_in, n_elements_limits, nTimes;
	const mwSize *dims;
	mwSize ndim;
	mxArray **Discrete_array;

	// the argument list is in the order - parameters,variables,nTimes,the formulas to be checked, the discretization of the variables, dt
	p_in = mxGetPr(prhs[0]);
	x_in = mxGetPr(prhs[1]);
	n_p_in = mxGetN(prhs[0]);
	n_x_in = mxGetN(prhs[1]);
	nTimes= (int)mxGetScalar(prhs[2]);
	n_formulas_in=mxGetNumberOfElements(prhs[3]);
	dt = mxGetScalar(prhs[4]);

	//convert the initial x values to vectors
	x_pre = new double[n_x_in];
	x_post = new double[2*n_x_in];

	memcpy(x_pre,x_in,n_x_in*sizeof(double));

	//string vector to read in the list of formulas
	vector<string> formula_list;
	vector<string>::iterator a;

	//create a vector of objects of MC type

	vector<ModelChecker*> MC;
	vector<int> tf;
	
	//printf("in");
	for (int i=0;i<n_formulas_in;i++)
	{
		formula_list.push_back(mxArrayToString(mxGetCell(prhs[3], i)));
	}
	

	int timePoint=1;

	//make the model checker instances for all formulas
	for (i=0;i<n_formulas_in;i++)
	{
		ModelChecker* M_temp = new ModelChecker(formula_list[i]);
		MC.push_back(M_temp);
		int tfi = MC[i]->update(x_in,0);
		tf.push_back(tfi);
	}
	
	//printf(".");
	vector<int>::iterator truthhood_iterator;
	int tLim = nTimes+2;
	
	double ts[2]={0.0,dt};
	double* lastState; 
    while ((find(tf.begin(),tf.end(),-1)!=tf.end()) && (timePoint < tLim) )
	{
		//simulate the pathway        
		int nStepsTaken = cvode_sim(ts,2,x_pre,p_in,odefun,x_post);
		lastState = x_post+n_x_in;
		memcpy(x_pre,lastState,n_x_in*sizeof(double));
		
		for (int i=0;i<n_formulas_in;i++)
		{
			if (tf[i]==-1)
			{
				tf[i]= MC[i]->update(x_post,timePoint==nTimes);
			}
		}
		timePoint=timePoint+1;
    }
	
	if (timePoint>=tLim){
		for(int i=0;i<tf.size();i++){
			printf("TF %d: %d\n",i,tf[i]);
			}
	}

	//create a matrix which returns the truthhood of the formulas.
	plhs[0]=getMexArray(tf);
	nlhs = mxGetM(plhs[0]);

	for(vector<ModelChecker*>::iterator it=MC.begin();it!=MC.end();++it){
		delete *it;
		}
	MC.clear();
	
	delete(x_pre);
	delete(x_post);

	return;
}

mxArray * getMexArray(const std::vector<int>& inputVector){
	mxArray * mexArray = mxCreateDoubleMatrix(1,inputVector.size(), mxREAL);
	copy(inputVector.begin(), inputVector.end(), mxGetPr(mexArray));
	return mexArray;
}